;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x93eb654a9f3d2e26d1efd79d7c34f81d . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x0142f7601c9ed27f522afab4a15d3eeb . "./code.cg2js/cg-gesture-emulation.fasl")
(#x562fbf4050ca47af0da8553df886aa81 . "./code.cg2js/cg-grid-widget.fasl")
(#x2a34858fc23ea1e2e4e04de5149524c0 . "./code.cg2js/cg-utility-dialog.fasl")
(#x94edca3595ac348852f1a227b8fdc15f . "./code.cg2js/cg-wrap-string.fasl")
(#x5c7a5e2b5bde8c4e4417a378b0ea1e34 . "./code.cg2js/cg-editable-text.fasl")
(#x9a0c55a2428f7755e41f7999e69910c7 . "./code.cg2js/cg-base.fasl")
(#x77a61dab19921adffc0c14c35b5d935d . "./code.cg2js/cg-lisp-text.fasl")
(#x66bc64e071b581d4f94ac8d76d26decd . "./code.cg2js/cg-intersect.fasl")
(#x905f0ccc2080c639bfe2277fbdb1d51e . "./code.cg2js/cg-choose-printer.fasl")
(#x709fc38c0d5b94f1f8d7260dd3c80183 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x01f74be3393ea6f9fc4738a254779871 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x6ac276c5e525d84c5c3d7f34342588d7 . "./code.cg2js/cg-font-dialog.fasl")
(#x4fac98f4da9acbac99b32a4a2e6a7339 . "./code.cg2js/cg-comtab.fasl")
(#xee224ee8e2fa1c881a457b7241ee365b . "./code.cg2js/cg-tooltip.fasl")
(#xa2d2a5813704f3d2c7ac2d0f0b506440 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x21ecb3098e7c0650cbd1871307ce3f4b . "./code.cg2js/cg-get-pixmap.fasl")
(#x379d7d8583d1110d6286fba27c4fa96d . "./code.cg2js/cg-status-bar.fasl")
(#x81e89d6cf288a25ec30a4fe6ad26d139 . "./code.cg2js/cg-tab-control.fasl")
(#xe29de184b8babafaeefee2e5af42de0a . "./code.cg2js/cg-icon-pixmap.fasl")
(#xe3c2c56c0c0f846ba1f4b86163b21fbc . "./code.cg2js/cg-list-view.fasl")
(#x07a170e86c068d0e842c72579a5ec2f5 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x904cb8a184c0fa7e848de7e32d49ad67 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xa8324c8b7cdadae08ecc423ba8a31e2f . "./code.cg2js/cg-project-window.fasl")
(#xc216906bf1f1ec3b7e8f56df95820c86 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xef74793fb55de09048a9aa57d09af13f . "./code.cg2js/cg-drag-and-drop.fasl")
(#x5525c6e1e2a73589a164b8becb2de447 . "./code.cg2js/cg-scaling-stream.fasl")
(#xf2590e1fca47d9490caabc32912b69a4 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xc188a978dd74371526fa1f498eba2fe1 . "./code.cg2js/cg-menu.fasl")
(#xecf479fbe8331c607690785cdac61e3b . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xdd60ad6c45c63ff327247de712a78878 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x2b3eabfa9d1cedf20abc1a11e1abe57d . "./code.cg2js/cg-split-bar.fasl")
(#x44828add3726f9c4a3e365131a76234c . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x751c73138a1b407c7ed2a6503eeed869 . "./code.cg2js/cg-timer.fasl")
(#x4fadb6901df677078e7d981446d7ec3a . "./code.cg2js/cg-text-or-combo.fasl")
(#xc7ca7d2f57b211b057477f5f318525ef . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xe37c80f4c31832d82b51b31b4b0b11fa . "./code.cg2js/cg-string-dialog.fasl")
(#x0f0e6c420a201c346f55ce3403457403 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xfe18b1745e868b5844e46a222cb911ae . "./code.cg2js/cg-menu-tooltip.fasl")
(#xbc1e5ad7b9df1d9a77cc0b40bbd1c8c4 . "./code.cg2js/cg-outline.fasl")
(#x5632af59efd29b53bca4f08192c5bd9c . "./code.cg2js/cg-lettered-menu.fasl")
(#xe75ac5cf676ab5f6973a2e22fca02d6f . "./code.cg2js/cg-graphics-context.fasl")
(#x4f90ea3640ec80e01c17429aa8908762 . "./code.cg2js/cg-item-list.fasl")
(#x1007d4608ac6e2ea859384936f2367c3 . "./code.cg2js/cg-caret.fasl")
(#xeb50d0541e6fc841a5053ecc9ca40361 . "./code.cg2js/cg-printing.fasl")
(#x16469982673fed390e59f9393043fe9e . "./code.cg2js/cg-pixmap-printing.fasl")
(#xfba5c612fed1a3e7ce32817de9ca124d . "./code.cg2js/cg-progress-indicator.fasl")
(#x31322ec833190e82cf80e0f22d5be998 . "./code.cg2js/cg-dde-utils.fasl")
(#xe73e8e16788221236b994d213f9eaae0 . "./code.cg2js/cg-plot-widget.fasl")
(#x9c2e9315b0fdad4a442e6e8083a1511e . "./code.cg2js/cg-os-window.fasl")
(#x8bc7d440a3036474e0e0c118356f2f71 . "./code.cg2js/cg-calendar.fasl")
(#xafcc79034b8dc53808ea604be4a5db79 . "./code.cg2js/cg-chart-widget.fasl")
(#x006bee2ae772b47400416bfc0c60b83f . "./code.cg2js/cg-file-dialog.fasl")
(#x629e1800320b31b8c8d30fdca14b43c6 . "./code.cg2js/cg-nodes.fasl")
(#x3b06ab49e962ac82baf20e3737bcede3 . "./code.cg2js/cg-lamp.fasl")
(#x3d0bf4daa80630c98ad9df64f55f4b8d . "./code.cg2js/cg-html-dialog.fasl")
(#x801ba6d3b7a52829ce3258a641c16cf2 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x75825250991ed75689ffbd022da33c2c . "./code.cg2js/cg-class-slot-grid.fasl")
(#xa95451fb4cac06c3f5fa2bfc41e2028c . "./code.cg2js/cg-choice-list.fasl")
(#x42fe0e52c5b957103c1d568c3f2fc41c . "./code.cg2js/cg-multi-picture-button.fasl")
(#xc7bf2bcb48573ada5815ad6869dc4896 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x00df8c717c864feb7e332286e0ea6c98 . "./code.cg2js/cg-hotspot.fasl")
(#xa200b5d29750af39ae90f24e1f0c39dc . "./code.cg2js/cg-property.fasl")
(#x0283e9a1379f944e3bee097a670a0651 . "./code.cg2js/cg-dropping-outline.fasl")
(#x2be539c4b8eeb50cb8262a64bd3837c1 . "./code.cg2js/cg-template-string.fasl")
(#x041f6c7ad825e48f8877c31a1cec8c08 . "./code.cg2js/cg-palette.fasl")
(#x8aa12099ebc08eb1d0b9cc3e2b79ea33 . "./code.cg2js/cg-selected-object.fasl")
(#x41b0caf0a02f753d940e2322953daa15 . "./code.cg2js/cg-up-down-control.fasl")
(#xeb3a192d390385fdda0a35019fbe74e5 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x9c786ee44d60a97633c376cff416fde1 . "./code.cg2js/cg-layout.fasl")
(#xd5f233a74058b73642664c0f535fc84a . "./code.cg2js/cg-get-position.fasl")
(#x5aa0bae1c089e4ff030bbec844eadd02 . "./code.cg2js/cg-lisp-widget.fasl")
(#xf37f07ce93cdb21e1cb114deabc5d3d3 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xea612c5322495ae0c33748c0d3a3dee6 . "./code.cg2js/cg-object-editor.fasl")
(#xb5d7b11da3673170fd441c8c1e9c298e . "./code.cg2js/cg-class-grid.fasl")
(#x6a2502873be807f13b327bbbe229e9fc . "./code.cg2js/cg-yes-no-string.fasl")
(#x89b9351ffcca0b9e94bc99c91ad239c1 . "./code.cg2js/cg-toggling-widget.fasl")
(#xe933ee1b9a3762fe6bba6ee0b7c45496 . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x4ce5799340dfb4134cd21f76c66a7b52 . "./code.cg2js/cg-button.fasl")
(#x0412cb01e7b8cfc7f7a598f61462a289 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x15c67d8cf6f22574184980b5d013def4 . "./code.cg2js/cg-scroll-bar.fasl")
(#x52198c12c14e9db8a29d3391c0120263 . "./code.cg2js/cg-combo-box.fasl")
(#xefc366a769d67a3cceed87614fde01d7 . "./code.cg2js/cg-web-browser.fasl")
(#x90fa37c1819bc18936c5120f6ce7e973 . "./code.cg2js/cg-drawable.fasl")
(#x64537c51126d5b08497e6e7850ac5352 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x62c0307ad76a7a5a43e06753450fbde1 . "./code.cg2js/cg-message-dialog.fasl")
(#x566c3b61831a2d1096f4941e0eabb8cb . "./code.cg2js/cg-text-widget.fasl")
(#x04169b298328fcc4f1e2da3b26f78a2f . "./code.cg2js/cg-static-text.fasl")
(#x4fd7f7f4c59eb1db9e6e035aa78193b1 . "./code.cg2js/cg-class-support.fasl")
(#xc532cbb432646eb85a290ecb3eee3f3f . "./code.cg2js/cg.fasl")
(#xa1cc79a13d07ed0c6c3a86bdf27dceaa . "./code.cg2js/cg-directory-dialog.fasl")
(#x239c6f8dfd4c97daa320012ee94d749e . "./code.cg2js/cg-clipboard.fasl")
(#x188b3255ab4bc9537bd3b0f5f6026668 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x33006e182a72c72772acd5198b27c0c9 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x9dd919f76dc6022d2f48095930cc53e6 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x61454418926d45e14a58b0b1632bfbf7 . "./code.cg2js/cg-check-box.fasl")
(#xcac5c8b5d61941d7fd3a3293a0639a92 . "./code.cg2js/cg-color-dialog.fasl")
(#x127f2ac043928499f3e6403d7630f1a5 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x1b64ed5fa52e9f273a6b88973be02e5e . "./code.cg2js/cg-dialog-item.fasl")
(#x88647a24e5f85d9a6be3ad2c6497a4fc . "./code.cg2js/cg-edit-in-place.fasl")
(#xd4fc8a33bf94d9727a6c26d65bb8537d . "./code.cg2js/cg-tray.fasl")
(#xd91779b9ef1cf0f8e76fbf1a23e08e81 . "./code.cg2js/cg-paren-matching.fasl")
(#xf7b5b468f920695b55fc713b35ab9c23 . "./code.cg2js/cg-player.fasl")
(#xa14ba0143ab654807c18f911480ca363 . "./code.cg2js/cg-yes-no-list.fasl")
(#xd9ddb1f36166b058985044c839365efa . "./code.cg2js/cg-common-control.fasl")
(#x8a3d05713bb5028f15e87e6f1a9267e1 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xb9407e96e8aafa3985cf5f954e63201b . "./code.cg2js/cg-os-widget.fasl")
(#x138de17ba23e08125ca5a744c4e7bdc7 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xf2eba89a273d47ea577f83aac96a6483 . "./code.cg2js/cg-trackbar.fasl")
(#x49df775d1f210cee51eb5c1be92cb9d5 . "./code.cg2js/cg-find-string-dialog.fasl")
(#xe0a18ada7aa155c540105071705b0002 . "./code.cg2js/cg-html-widget.fasl")
(#x75f19b5083c46ea212dae7f8c2d21d7b . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x8b9edd4a81b965d4cfce66d24d481a54 . "./code.cg2js/cg-acache.fasl")
(#xec1a59a75110bbe85aec24cf9a797c57 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x5e2d297f476fdfc7ae47e0afea345212 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x1b10a937914a3d137d8e4cdb658e1933 . "./code.cg2js/cg-icon.fasl")
(#x36227d91765c9dc4fd3e36fc606bf93d . "./code.cg2js/cg-pixmap.fasl")
(#x8d9d9a3aaffcd1505caddf9925da44da . "./code.cg2js/cg-clipboard-stack.fasl")
(#xd66a934472254d85f632d56a99333019 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xfda0f981fb3c6dc91c9ff4361b345a1d . "./code.cg2js/cg-toolbar.fasl")
(#x37aed7e26f4ddc21d051c0bdfaf16516 . "./code.cg2js/cg-radio-button.fasl")
(#xa488434d2b3dc7f0b2a2caf1e149a118 . "./code.cg2js/cg-curve.fasl")
(#x84984367a1182c791d938527255a83b9 . "./code.cg2js/cg-picture-widget.fasl")
(#x6fc4d6b79370b7f7323f4b56c4c38355 . "./code.cg2js/cg-dev.fasl")
(#x2cd5cc8364805a9bd1202869adee8f5c . "./code.cg2js/cg-group-box.fasl")
(#x84ce5eda905952dd8db39e51ff960c89 . "./code.cg2js/cg-fill-texture.fasl")
(#x0941804f4df6dc5e1de625e8fc3c9766 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x7d11d787d5e5dd640e12c711f618e186 . "./code.idejs/ide-keyshort.fasl")
(#x208dbaaf6376be0ddcd64b55c17a308c . "./code.idejs/ide-form.fasl")
(#x32e071218ce98fcf82dd2620eff6ab23 . "./code.idejs/ide-find-in-files.fasl")
(#xc001fe9f3e783c9a1941768912e45b1d . "./code.idejs/ide-class-grapher.fasl")
(#x7998c8316c1020c886499982c797cffb . "./code.idejs/ide-base.fasl")
(#xa4fab5eadede21050e40be742f58ec30 . "./code.idejs/ide-class-browser.fasl")
(#xab127cdb48c11b6f768679778a852511 . "./code.idejs/ide-find-definitions.fasl")
(#x0bee204024c37239e00cfae9bf469318 . "./code.idejs/ide-package-list.fasl")
(#x396cd0cad3cafb433add4d849f778137 . "./code.idejs/ide-grapher.fasl")
(#x01a4e310347cbbe6ee93714cb51365be . "./code.idejs/ide-dev.fasl")
(#x8e1197d39302c24a95b5239557acf477 . "./code.idejs/ide.fasl")
(#x71a2cdf93fcd8b7b40d01cdadb9ebc09 . "./code.idejs/ide-trace.fasl")
(#x937ec32fb9b250ddb919c2a629c39ddc . "./code.idejs/ide-apropos.fasl")
(#x34de2de628e8a21f7416b9c4ae7a7e6c . "./code.idejs/ide-inspector.fasl")
(#x36fede89265f6d93fa2ecb1f663ecf2c . "./code.idejs/ide-acache.fasl")
(#x0df44847fc540f80a8a00af434ab39ed . "./code.idejs/ide-project.fasl")
(#x7c3bf1ae731ee31f9acd78daf1ed13b1 . "./code.idejs/ide-window-list.fasl")
(#x6b0d6b1c57e105c7b8a73dbf9d6f6bbf . "./code.idejs/ide-intro.fasl")
(#x40bc8a19d46a07bb1c0c09fea68f56c8 . "./code.idejs/ide-clipboard.fasl")
(#x7bfbb737232d245f83235759f78d1f7b . "./code.idejs/ide-editor.fasl")
(#x30e80ba9330dc4e0888c8cf8b74fcb20 . "./code.idejs/ide-process-browser.fasl")
(#x4b4e7f35d127dc49c8a67ba743e42c12 . "./code.idejs/ide-patch-dialog.fasl")
(#x0bb8162843fc3973a45aba4d5c7d9831 . "./code.idejs/ide-navigator.fasl")
(#x4a9bad5a7becdca3247e09ac504560a1 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
